import * as React from 'react';
import {observer, Observer} from "mobx-react-lite";
import {UserModel} from "../../stores/UserStore";
import ReactTable from "react-table";
import {t} from "@lingui/macro";
import {CurrentUserModel, OAuthModel, OAuthStore, SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";
import CheckCircle from "@material-ui/icons/CheckCircle";
import Error from "@material-ui/icons/Error";
import {green, red} from "@material-ui/core/colors";
import {ButtonField} from "@rx-suite/ui";
import {useSnackbar} from "notistack";
import {errToString} from "@rx-suite/data";

export const OAuthsTable: React.FunctionComponent<{ user: UserModel | CurrentUserModel, readOnly?: boolean }> = observer(({
                                                                                                                              user,
                                                                                                                              readOnly
                                                                                                                          }) => {
    const {enqueueSnackbar} = useSnackbar();
    React.useEffect(() => {
        if (user) {
            user.getOAuths();
        }
    }, [user]);

    if (!user) {
        return null;
    }

    const columns = [
        {
            Header: t`Service`,
            id: 'service_name',
            accessor: model => model,
            Cell: (row: { original: OAuthModel | null | undefined }) => {
                return (
                    <Observer>
                        {
                            () => {
                                return <div>
                                    {row.original?.service_name || ''}
                                </div>
                            }
                        }
                    </Observer>
                )
            },
        },
        {
            Header: t`Account`,
            id: 'account_identifier',
            accessor: model => model,
            Cell: (row: { original: OAuthModel | null | undefined }) => {
                return (
                    <Observer>
                        {
                            () => {
                                return <div>
                                    {row.original?.account_identifier || ''}
                                </div>
                            }
                        }
                    </Observer>
                )
            },
        },
        {
            Header: t`Refresh Time`,
            id: 'last_refresh_time',
            accessor: model => model,
            Cell: (row: { original: OAuthModel | null | undefined }) => {
                return (
                    <Observer>
                        {
                            () => {
                                const date = row.original?.last_refresh_time as Date | null;
                                const systemSettings = SystemSettingStore.selectedModel as SystemSettingModel;
                                let formatted = "";
                                if (date) {
                                    formatted = systemSettings.printDateTime(date);
                                }
                                return <div>
                                    {formatted}
                                </div>
                            }
                        }
                    </Observer>
                )
            },
        },
        {
            Header: t`Refresh Result`,
            id: 'last_refresh_result',
            accessor: model => model,
            Cell: (row: { original: OAuthModel | null | undefined }) => {
                return (
                    <Observer>
                        {
                            () => {
                                if (!row.original) {
                                    return null;
                                }
                                if (row.original.last_refresh_success) {
                                    return <CheckCircle style={{color: green[500]}}/>
                                } else {
                                    return <div>
                                        <Error style={{color: red[500]}}/>
                                        {row.original.last_refresh_message}
                                    </div>
                                }
                            }
                        }
                    </Observer>
                )
            },
        },
        {
            Header: t`Selected`,
            id: 'selected',
            accessor: model => model,
            Cell: (row: { original: OAuthModel | null | undefined }) => {
                return (
                    <Observer>
                        {
                            () => {
                                if (!row.original) {
                                    return null;
                                }
                                if (row.original.selected) {
                                    return <CheckCircle style={{color: green[500]}}/>
                                }
                                if (readOnly) {
                                    return <Error style={{color: red[500]}}/>
                                } else {
                                    return <ButtonField
                                        text={t`Select`}
                                        loading={row.original.updating}
                                        onClick={async () => {
                                            try {
                                                await row.original!.activate();
                                            } catch (e) {
                                                enqueueSnackbar(t`Issue selecting account: ${errToString((e))}`, {variant: 'warning'})
                                            }
                                        }}
                                    />
                                }
                            }
                        }
                    </Observer>
                )
            },
        }
    ];
    if (!readOnly) {
        columns.push({
            Header: '',
            id: 'delete',
            accessor: model => model,
            Cell: (row: { original: OAuthModel | null | undefined }) => {
                return (
                    <Observer>
                        {
                            () => {
                                if (!row.original) {
                                    return null;
                                }
                                return <ButtonField
                                    text={t`Disconnect`}
                                    loading={row.original.updating}
                                    onClick={async () => {
                                        try {
                                            await row.original!.disconnect();
                                        } catch (e) {
                                            enqueueSnackbar(t`Issue disconnecting service: ${errToString((e))}`, {variant: 'warning'})
                                        }
                                    }}
                                />
                            }
                        }
                    </Observer>
                )
            },
        })
    }

    const data = user.oauths.slice();
    data.sort((a, b) => {
        const aService = a.service_name || '';
        const bService = b.service_name || '';
        const aAccount = a.account_identifier || '';
        const bAccount = b.account_identifier || '';
        if (aService !== bService) {
            return aService.localeCompare(bService);
        }
        return aAccount.localeCompare(bAccount);
    });

    return <ReactTable
        data={data}
        columns={columns}
        loading={OAuthStore.isLoading}
        sortable={false}
        filterable={false}
    />
});