import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";
import {UsersTransport} from "../transports/UsersTransport";
import {action, observable, runInAction} from "mobx";
import {OAuthStore} from "@rx-suite/stores";

//plucked from yup library
// eslint-disable-next-line no-control-regex,no-useless-escape
const rEmail = /^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))$/i; // eslint-disable-next-line
export const isEmail = (input: string) => {
    return rEmail.test(input);
}

@model({
    transportLayer: new UsersTransport(),
    permissionTarget: "User",
    routingInfo: {
        list: '/users',
        create: '/users/create',
        read: '/users/:id/read',
        update: '/users/:id/edit'
    }
})
export class UserModel extends Model {
    constructor() {
        super();
        super.setup();
    }

    @observable hasOauths = false;
    @observable fetchingOauths = false;


    @attribute({
        label: t`Username`,
        description: t`The username the user will use to log in`,
        validation: yup.string().required(t`Username is a required field`)
    })
    name = '';

    @attribute({
        label: t`Email`,
        description: t`The user can use this address to log in; it will also be used for sending messages to the user.`,
        validation: yup.string().nullable().test('valid-email', t`If provided, email must be valid`, (val) => {
            return !val || (val === 'admin') || isEmail(val)
        })
    })
    email = '';

    @attribute({
        label: t`Table Size`,
        description: t`The default table size used throughout Rx-Suite`
    })
    table_size = 10

    @attribute({
        label: t`Role`,
        description: t`This defines the functions the user is allowed to use.`,
        connects: 'RoleModel',
        validation: yup.number().required(t`Users must be assigned a Role`),
    })
    role_id: number | undefined = undefined;

    @action.bound
    sendPasswordResetLink() {
        return (this.store.transportLayer as UsersTransport).sendPasswordResetLink(this.getId())
    }

    @action.bound
    async getPasswordResetLink() {
        const resp = await (this.store.transportLayer as UsersTransport).getPasswordResetLink(this.getId()) as { link: string } | undefined;
        if (resp) {
            return resp.link;
        }
    }

    get oauths() {
        this.getOAuths();
        return OAuthStore.models.filter(model => this.getId().toString() === (model.user_id.toString()));
    }

    @action
    async getOAuths(force = false) {
        if (!this.fetchingOauths) {
            if (force || !this.hasOauths) {
                try {
                    runInAction(() => {
                        this.fetchingOauths = true;
                    });
                    await OAuthStore.getOAuths(this.getId());
                    runInAction(() => {
                        this.hasOauths = true;
                    });
                } finally {
                    runInAction(() => {
                        this.fetchingOauths = false;
                    })
                }
            }
        }
    }
}

export const UserStore = UserModel.store as unknown as DataStore<UserModel>;
